#Eneroth Viewport Resizer

#Copyright Julia Christina Eneroth (eneroh3)

module Ene_ViewprtResizer

#Require win32api
if RUBY_VERSION < "2.0.0"
  require File.join(PLUGIN_ROOT, "ene_viewport_resizer", "Win32API.so")#Bundled with plugin for SU 2013.
else
  require "Win32API"#Standard libraries included in SU 2014+.
end  

GetActiveWindow = Win32API.new("user32.dll", "GetActiveWindow", "", "L")
GetWindowRect = Win32API.new("user32.dll", "GetWindowRect", "LP", "I")
ShowWindow = Win32API.new("user32.dll", "ShowWindow", "LI", "I")
MoveWindow = Win32API.new("user32.dll", "MoveWindow", "LIIIII", "I")

def self.get_rect(hwnd)
  #Returns array of left, top, right and bottom

  rect = [0, 0, 0, 0].pack("L*")
  GetWindowRect.call hwnd, rect
  
  rect.unpack("L*").map { |e| [e].pack("L").unpack("l").first }
  
end#def

def self.get_size(hwnd)
  #Get window size as array of width and height.

  left, top, right, bottom = self.get_rect hwnd

  w = right - left
  h = bottom - top
  
  [w, h]
  
end#def

def self.set_size(hwnd, size)
  #Set window size from array of width and height.
  
  #'restore window' (stop it from being maximized)
  #Without this the window would still get the wanted size be be 'marked' as maximized and moving it would result in reseting the size (Win7).
  ShowWindow.call hwnd, 9;#9 = SW_RESTORE
  
  left, top, right, bottom = self.get_rect hwnd
  MoveWindow.call hwnd, left, top, size[0], size[1], 1
  
end

def self.resize(wp = true, size = nil)
  #wp tells if viewport should be resized instead of whole window
  #Size is an array containing 2 integers, the first is width, the second height.
  #Size is only for debug purposes and will make an incorrect window being resized since it is active instead of the main SU window.

  #Assume the main Sketchup window is the active one
  hwnd = GetActiveWindow.call
  
  #User input
  unless size
    prompts = ["Width", "Height"]
    if wp
      title = "Resize Viewport"
      view = Sketchup.active_model.active_view
      defaults = [view.vpwidth, view.vpheight]
    else
      title = "Resize Window"
      defaults = self.get_size hwnd
    end
    input = UI.inputbox prompts, defaults, title
    return unless input
    size = input
  end
  size.each do |i|#Set minimum window size since window behaves oddly when to small.
    next if i >= 100
    UI.messagebox "Cannot be made smaller than 100 px in any direction."
    return
  end
  
  if wp
    #Set viewport size
    
    #Loop this code a few times since toolbars can get 'thicker' when made narrower and change delta_size value.
    3.times do
  
      #Get current window size.
      window_size = self.get_size hwnd
      
      #Get current viewport size.
      view = Sketchup.active_model.active_view
      wp_size = [view.vpwidth, view.vpheight]
      
      #Calculate size toolbars and window frame takes up.
      delta_size = window_size.zip(wp_size).map { |x, y| x - y }
      
      #Calculate new window size.
      window_new_size = size.zip(delta_size).map { |x, y| x + y }
      
      #Stop loop if window size is already correct.
      break if window_size == window_new_size
      
      self.set_size hwnd, window_new_size
    
    end
    
  else
    #Set whole window's size
    
    self.set_size hwnd, size
    
  end
  
  true
  
end#def

#Menus and toolbars
file = __FILE__
unless file_loaded? file

  menu = UI.menu("Plugins")

  menu.add_item("Resize Window") { self.resize false}
  menu.add_item("Resize Viewport") { self.resize true}

  file_loaded file
end

end#module